@@ -27,7 +27,7 @@
 #include <QApplication>
 #include <QDebug>
 #include <QPainter>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QSize>
 #include <QString>
 #include <cstring>
@@ -59,7 +59,7 @@ GmicProcessor::GmicProcessor(QObject * parent) : QObject(parent)
   gmic_library::cimg::srand();
   _previewRandomSeed = gmic_library::cimg::_rand();
   _lastAppliedCommandInOutState = InputOutputState::Unspecified;
-  _filterExecutionTime.start();
+  _ongoingFilterExecutionTime.start();
   _completeFullImageProcessingCount = 0;
 }
 
@@ -69,6 +69,16 @@ void GmicProcessor::init()
   _gmicImages->assign();
 }
 
+GmicProcessor::~GmicProcessor()
+{
+  delete _gmicImages;
+  delete _previewImage;
+  if (!_unfinishedAbortedThreads.isEmpty()) {
+    Logger::error(QString("~GmicProcessor(): There are %1 unfinished filter threads.").arg(_unfinishedAbortedThreads.size()));
+    detachAllUnfinishedAbortedThreads();
+  }
+}
+
 void GmicProcessor::setContext(const GmicProcessor::FilterContext & context)
 {
   _filterContext = context;
@@ -137,6 +147,7 @@ void GmicProcessor::execute()
   env += QString(" _preview_y1=%1").arg(preview_y1);
   env += QString(" _preview_width=%1").arg(previewSize.width());
   env += QString(" _preview_height=%1").arg(previewSize.height());
+  _completedExecutionTime.restart();
   if (_filterContext.requestType == FilterContext::RequestType::SynchronousPreview) {
     FilterSyncRunner runner(this, _filterContext.filterCommand, _filterContext.filterArguments, env);
     runner.swapImages(*_gmicImages);
@@ -144,10 +155,10 @@ void GmicProcessor::execute()
     runner.setLogSuffix("preview");
     gmic_library::cimg::srand();
     _previewRandomSeed = gmic_library::cimg::_rand();
-    _filterExecutionTime.restart();
+    _ongoingFilterExecutionTime.restart();
     runner.run();
     manageSynchonousRunner(runner);
-    recordPreviewFilterExecutionDurationMS((int)_filterExecutionTime.elapsed());
+    recordPreviewFilterExecutionDurationMS((int)_ongoingFilterExecutionTime.elapsed());
   } else if (_filterContext.requestType == FilterContext::RequestType::Preview) {
     _filterThread = new FilterThread(this, _filterContext.filterCommand, _filterContext.filterArguments, env);
     _filterThread->swapImages(*_gmicImages);
@@ -156,7 +167,7 @@ void GmicProcessor::execute()
     connect(_filterThread, &FilterThread::finished, this, &GmicProcessor::onPreviewThreadFinished, Qt::QueuedConnection);
     gmic_library::cimg::srand();
     _previewRandomSeed = gmic_library::cimg::_rand();
-    _filterExecutionTime.restart();
+    _ongoingFilterExecutionTime.restart();
     _filterThread->start();
   } else if (_filterContext.requestType == FilterContext::RequestType::FullImage) {
     _lastAppliedFilterHash = _filterContext.filterHash;
@@ -176,7 +187,7 @@ void GmicProcessor::execute()
 
 bool GmicProcessor::isProcessingFullImage() const
 {
-  return _filterContext.requestType == FilterContext::RequestType::FullImage;
+  return _filterThread && (_filterContext.requestType == FilterContext::RequestType::FullImage);
 }
 
 bool GmicProcessor::isProcessing() const
@@ -245,11 +256,44 @@ int GmicProcessor::completedFullImageProcessingCount() const
   return _completeFullImageProcessingCount;
 }
 
+qint64 GmicProcessor::lastCompletedExecutionTime() const
+{
+  return _lastCompletedExecutionTime;
+}
+
 void GmicProcessor::cancel()
 {
   abortCurrentFilterThread();
 }
 
+void GmicProcessor::detachAllUnfinishedAbortedThreads()
+{
+  for (FilterThread * thread : _unfinishedAbortedThreads) {
+    thread->disconnect(this);
+    thread->setParent(nullptr);
+  }
+  _unfinishedAbortedThreads.clear();
+}
+
+void GmicProcessor::terminateAllThreads()
+{
+  if (_filterThread) {
+    _filterThread->disconnect(this);
+    _filterThread->terminate();
+    _filterThread->wait();
+    delete _filterThread;
+  }
+  while (!_unfinishedAbortedThreads.isEmpty()) {
+    _unfinishedAbortedThreads.front()->disconnect(this);
+    _unfinishedAbortedThreads.front()->terminate();
+    _unfinishedAbortedThreads.front()->wait();
+    delete _unfinishedAbortedThreads.front();
+    _unfinishedAbortedThreads.pop_front();
+  }
+  _waitingCursorTimer.stop();
+  OverrideCursor::setWaiting(false);
+}
+
 bool GmicProcessor::hasUnfinishedAbortedThreads() const
 {
   return !_unfinishedAbortedThreads.isEmpty();
@@ -288,15 +332,6 @@ void GmicProcessor::saveSettings(QSettings & settings)
   }
 }
 
-GmicProcessor::~GmicProcessor()
-{
-  delete _gmicImages;
-  delete _previewImage;
-  if (!_unfinishedAbortedThreads.isEmpty()) {
-    Logger::error(QString("~GmicProcessor(): There are %1 unfinished filter threads.").arg(_unfinishedAbortedThreads.size()));
-  }
-}
-
 void GmicProcessor::setGmicStatusQuotedParameters(const QVector<bool> & quotedParameters)
 {
   _gmicStatusQuotedParameters = quotedParameters;
@@ -308,6 +343,7 @@ void GmicProcessor::onPreviewThreadFinished()
   if (_filterThread->isRunning()) {
     return;
   }
+  _lastCompletedExecutionTime = _completedExecutionTime.elapsed();
   if (_filterThread->failed()) {
     _gmicStatus.clear();
     _parametersVisibilityStates.clear();
@@ -337,7 +373,7 @@ void GmicProcessor::onPreviewThreadFinished()
   hideWaitingCursor();
   if (correctSpectrums) {
     emit previewImageAvailable();
-    recordPreviewFilterExecutionDurationMS((int)_filterExecutionTime.elapsed());
+    recordPreviewFilterExecutionDurationMS((int)_ongoingFilterExecutionTime.elapsed());
   } else {
     QString message(tr("Image #%1 returned by filter has %2 channels (should be at most 4)"));
     emit previewCommandFailed(message.arg(badSpectrumIndex).arg((*_gmicImages)[badSpectrumIndex].spectrum()));
@@ -351,6 +387,7 @@ void GmicProcessor::onApplyThreadFinished()
   if (_filterThread->isRunning()) {
     return;
   }
+  _lastCompletedExecutionTime = _completedExecutionTime.elapsed();
   _gmicStatus = _filterThread->gmicStatus();
   _parametersVisibilityStates = _filterThread->parametersVisibilityStates();
   hideWaitingCursor();
@@ -428,13 +465,14 @@ void GmicProcessor::updateImageNames(gmic_list<char> & imageNames)
   for (size_t i = 0; i < imageNames.size(); ++i) {
     gmic_image<char> & name = imageNames[i];
     QString str((const char *)name);
-    QRegExp position("pos\\((\\d*)([^0-9]*)(\\d*)\\)");
-    if (str.contains(position) && position.matchedLength() > 0) {
-      int xPos = position.cap(1).toInt();
-      int yPos = position.cap(3).toInt();
+    QRegularExpression re(R"_(pos\((\d*)([^0-9]*)(\d*)\))_");
+    auto match = re.match(str);
+    if (match.hasMatch() && match.captured(1).size() && match.captured(3).size()) {
+      int xPos = match.captured(1).toInt();
+      int yPos = match.captured(3).toInt();
       int newXPos = (int)(xPos * (xFactor / (double)maxWidth));
       int newYPos = (int)(yPos * (yFactor / (double)maxHeight));
-      str.replace(position.cap(0), QString("pos(%1%2%3)").arg(newXPos).arg(position.cap(2)).arg(newYPos));
+      str.replace(match.captured(0), QString("pos(%1%2%3)").arg(newXPos).arg(match.captured(2)).arg(newYPos));
       name.resize(str.size() + 1);
       std::memcpy(name.data(), str.toLatin1().constData(), name.width());
     }
@@ -457,6 +495,7 @@ void GmicProcessor::abortCurrentFilterThread()
 
 void GmicProcessor::manageSynchonousRunner(FilterSyncRunner & runner)
 {
+  _lastCompletedExecutionTime = _completedExecutionTime.elapsed();
   if (runner.failed()) {
     _gmicStatus.clear();
     _gmicImages->assign();
