@@ -89,7 +89,6 @@ public:
   void execute();
 
   bool isProcessingFullImage() const;
-
   bool isProcessing() const;
   bool isIdle() const;
   bool hasUnfinishedAbortedThreads() const;
@@ -108,9 +107,12 @@ public:
   void recordPreviewFilterExecutionDurationMS(int duration);
   int averagePreviewFilterExecutionDuration() const;
   int completedFullImageProcessingCount() const;
+  qint64 lastCompletedExecutionTime() const;
 
 public slots:
   void cancel();
+  void detachAllUnfinishedAbortedThreads();
+  void terminateAllThreads();
 
 signals:
   void previewCommandFailed(QString errorMessage);
@@ -150,7 +152,9 @@ private:
   QString _lastAppliedCommandArguments;
   QStringList _lastAppliedCommandGmicStatus;
   InputOutputState _lastAppliedCommandInOutState;
-  QElapsedTimer _filterExecutionTime;
+  QElapsedTimer _ongoingFilterExecutionTime;
+  QElapsedTimer _completedExecutionTime;
+  qint64 _lastCompletedExecutionTime;
   std::deque<int> _lastFilterPreviewExecutionDurations;
   int _completeFullImageProcessingCount;
   QVector<bool> _gmicStatusQuotedParameters;
